/*
 * Project name:
     PerfectEnvironment.mcpar
 * Generated by:
     Visual TFT
 * Date of creation
     3/4/2020
 * Test configuration:
     MCU:             STM32F407ZG
     Dev.Board:       Fusion_for_STM32_Board_4_Capacitive
     Oscillator:      168000000 Hz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 */

#include "PerfectEnvironment_objects.h"
#include "Adc.h"
#include "Demo_types.h"
#include "Demo_config.h"

#define __FAN4__
#define __PRESSURE6__
#define __AMBIENT5__
#define __UV__
#define __TEMPHUM6__
#define __WATERDETECT__
#define __AIRQUALITY4__

uint16_t Pressure;
uint8_t  Temperature;
uint16_t AIRQ3_Data[ 5 ] = { 0 };
char Pressure_str[ 50 ] = { 0 };
char Temperature_str[ 50 ] = { 0 };
char CO2_str[ 256 ] = { 0 };
char TVOC_str[ 256 ] = { 0 };
char Ambient_light_str[ 50 ] = { 0 };
char UV_str[ 250 ] = { 0 };
char Humidity_str[ 50 ] = { 0 };

uint16_t rTime;
uint16_t i;
uint16_t lth;
uint16_t hth;
float highResLightLevel;
float highThreshold = 500.0;
float lowThreshold = 100.0;

float uvData;
float uvDataOld = 1.0;
float sensitivity = 20.0;

uint16_t PartID;
float TEMP;
float HUM;

uint8_t wdState = 0;
uint8_t wdStateOld = 0;
uint8_t flag = 1;

uint16_t valueADC;

uint8_t buffer_data[ 6 ];
uint16_t data_buffer[ 2 ];

uint8_t button1_flag = 0;
uint8_t button1_active = 0;

uint8_t timer_flag = 0;

void decimal_trim( char * value, int num_of_dec )
{
     int i,n;
     n = strlen( value );
     for ( i=0; i<n; i++ )
     {
         if ( value[ i ]=='.' )
         {
             if ( num_of_dec == 0 )
             {
                 value[ i ]='\0';
             }
             else
             {
                 num_of_dec++;
                 value[ i + num_of_dec ]='\0';
             }
             break;
         }
     }
}

/** Formats float string from exponential to scientific notation with 3 decimals */
void formatFloatStr( char *s )
{
   uint8_t i;
   uint8_t point;
   uint8_t degree = 0;
   for( i = 0, point = 0; s[i] != '\0'; i++ )
   {
      if( s[i] == '.' ) {
          point = i;
          break;
      }
   }
   for( i = point + 1; s[i] != '\0'; i++ ) {
      if( s[i] == '-' ) {
          i++;
          while( s[i] == '0' ) {
             i++;
          }
          degree = s[i] - '0';
          break;
      }
   }
   if( degree > 4 ) {
      s[point - 1] = '0';
      s[point + 1] = '0';
      s[point + 2] = '0';
      s[point + 3] = '0';
      s[point + 4] = '\0';
   }
   else if( degree == 4 ) {
           if( s[point - 1] >= '5' ) {
              s[point - 1] = '0';
              s[point + 1] = '0';
              s[point + 2] = '0';
              s[point + 3] = '1';
              s[point + 4] = '\0';
           }
           else {
              s[point - 1] = '0';
              s[point + 1] = '0';
              s[point + 2] = '0';
              s[point + 3] = '0';
              s[point + 4] = '\0';
           }
   }
   else {
      switch(degree) {
         case 0: s[point + 4] = '\0';
                 break;
         case 1: s[point + 3] = s[point + 2];
                 s[point + 2] = s[point + 1];
                 s[point + 1] = s[point - 1];
                 s[point - 1] = '0';
                 s[point + 4] = '\0';
                 break;
         case 2: s[point + 3] = s[point + 1];
                 s[point + 2] = s[point - 1];
                 s[point + 1] = '0';
                 s[point - 1] = '0';
                 s[point + 4] = '\0';
                 break;
         case 3: s[point + 3] = s[point - 1];
                 s[point + 2] = '0';
                 s[point + 1] = '0';
                 s[point - 1] = '0';
                 s[point + 4] = '\0';
                 break;
         default: s[point + 4] = '\0';
                  break;
      }
   }
}

void turn_fan_on_and_off( )
{
    if ( button1_active == 1 )
    {
       if ( strcmp( Label14_Caption, "ON") == 0 )
       {
          strcpy(Label14_Caption, "OFF");
          fan4_setOutput( _FAN4_MIN_VOLT_SCALE, _FAN4_BOOST_START_TIMER_DIS );
       }
       else if ( strcmp( Label14_Caption, "OFF" ) == 0 )
       {
          strcpy( Label14_Caption, "ON" );
          fan4_setOutput( _FAN4_MAX_VOLT_SCALE, _FAN4_BOOST_START_TIMER_DIS );
       }

       if ( flag <= 1 )
       {
           DrawImage( &Image9 );
       }
       else if ( flag == 2 )
       {
            DrawImage( &Image10 );
       }
       else if ( flag >= 3 )
       {
            DrawImage( &Image11 );
       }
       else
           DrawImage( &Image9 );

       DrawButton( &Button1 );
       DrawLabel( &Label14 );
       button1_active = 0;
    }
}

void systemInit( )
{
    #ifdef __FAN4__
    mikrobus_i2cInit( _MIKROBUS1, &_FAN4_I2C_CFG[ 0 ] );
    Delay_ms( 10 );
    #endif

    #ifdef __PRESSURE6__
    mikrobus_gpioInit( _MIKROBUS5, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_i2cInit( _MIKROBUS5, &_PRESSURE6_I2C_CFG[ 0 ] );
    Delay_ms( 10 );
    #endif

    #ifdef __AMBIENT5__
    mikrobus_gpioInit( _MIKROBUS5, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_i2cInit( _MIKROBUS5, &_AMBIENT5_I2C_CFG[ 0 ] );
    Delay_ms( 10 );
    #endif

    #ifdef __UV__
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_AN_PIN, _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_CS_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_RST_PIN, _GPIO_OUTPUT );
    mikrobus_spiInit( _MIKROBUS4, &_UV_SPI_CFG[ 0 ] );
    Delay_ms( 10 );
    #endif

    #ifdef __TEMPHUM6__
    mikrobus_i2cInit( _MIKROBUS3, &_TEMPHUM6_I2C_CFG[ 0 ] );
    #endif

    #ifdef __WATERDETECT__
    mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    #endif

    #ifdef __AIRQUALITY4__
    mikrobus_i2cInit( _MIKROBUS2, &_AIR_QUALITY4_I2C_CFG[ 0 ] );
    Delay_ms( 10 );
    #endif

    mikrobus_logInit( _LOG_USBUART, 9600 );
}

void applicationInit( )
{
    #ifdef __FAN4__
    fan4_i2cDriverInit( (T_FAN4_P)&_MIKROBUS1_GPIO, (T_FAN4_P)&_MIKROBUS1_I2C, _FAN4_SLAVE_ADDR );
    Delay_ms( 10 );
    fan4_setOutput( _FAN4_MIN_VOLT_SCALE, _FAN4_BOOST_START_TIMER_DIS );
    #endif

    #ifdef __PRESSURE6__
    pressure6_i2cDriverInit( (T_PRESSURE6_P)&_MIKROBUS5_GPIO, (T_PRESSURE6_P)&_MIKROBUS5_I2C, _PRESSURE6_DEF_SLAVE_ADDRESS );
    pressure6_powerON( );
    Delay_1sec( );
    pressure6_writeByte(_PRESSURE6_REG_MODE_CONTROL,
                        _PRESSURE6_MC_AVE_NUM_SINGLE |
                        _PRESSURE6_MC_DATA_READY_ENABLED |
                        _PRESSURE6_MC_FULL_DATA_ENABLED |
                        _PRESSURE6_MC_WATER_MARK_ENABLED |
                        _PRESSURE6_MC_MODE_CONTINUOUS);
    #endif

    #ifdef __AMBIENT5__
    ambient5_i2cDriverInit( (T_AMBIENT5_P)&_MIKROBUS5_GPIO, (T_AMBIENT5_P)&_MIKROBUS5_I2C, _AMBIENT5_DEVICE_ADDRESS );
    Delay_ms( 10 );
    ambient5_powerOn( );
    Delay_ms( 10 );

    ambient5_setLowLightLevelThreshold( lowThreshold );
    ambient5_setHighLightLevelThreshold( highThreshold );

    rTime = ambient5_getRefreshTime( );
    #endif

    #ifdef __UV__
    uv_spiDriverInit( (T_UV_P)&_MIKROBUS4_GPIO, (T_UV_P)&_MIKROBUS4_SPI );
    uv_enableSensor( );
    Delay_ms( 10 );
    #endif

    #ifdef __TEMPHUM6__
    temphum6_i2cDriverInit( (T_TEMPHUM6_P)&_MIKROBUS3_GPIO, (T_TEMPHUM6_P)&_MIKROBUS3_I2C, _TEMPHUM6_DEVICE_SLAVE_ADDRESS );
    temphum6_reset( );
    PartID = temphum6_getPartID( );
    #endif

    #ifdef __WATERDETECT__
    waterdetect_gpioDriverInit( (T_WATERDETECT_P)&_MIKROBUS3_GPIO );
    Delay_ms( 10 );
    #endif

    #ifdef __AIRQUALITY4__
    air_quality4_i2cDriverInit( (T_AIR_QUALITY4_P)&_MIKROBUS2_GPIO, (T_AIR_QUALITY4_P)&_MIKROBUS2_I2C, 0x58 );
    Delay_ms( 10 );

    air_quality4_init( );
    air_quality4_getID( &buffer_data[ 0 ] );
    #endif
}

void applicationTask( )
{
#ifdef __PRESSURE6__
    pressure6_waitingForNewData( );
    Pressure = pressure6_getPressure( );
    Temperature = pressure6_getTemperatre( );

    IntToStr( Pressure, Pressure_str );
    DrawImage( &Image5 );
    Label10.Caption = Pressure_str;
    DrawLabel( &Label10 );
#endif

#ifdef __AMBIENT5__
    highResLightLevel = ambient5_getHighResolutionLightLevel( );
    FloatToStr( highResLightLevel, Ambient_light_str );
    decimal_trim( Ambient_light_str, 1 );
    DrawImage( &Image4 );
    Label9.Caption = Ambient_light_str;
    DrawLabel( &Label9 );
#endif

#ifdef __UV__
    uvData = uv_calculateUV( );   // ADC value
    /*
    Conversion from mV to mW/cm2
    float mapfloat(float x, float in_min, float in_max, float out_min, float out_max)
    {
          return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
    }
    float uvIntensity = mapfloat(outputVoltage, 0.98, 2.8, 0.0, 15.0); //Convert the voltage to a UV intensity level
    */

    uvData /=1000;
    uvData -= 0.98;
    uvData *= 8.24;

    FloatToStr( uvData, UV_str );  // mW/cm2
    formatFloatStr( UV_str );
    DrawImage( &Image6 );
    Label11.Caption =  UV_str;
    DrawLabel( &Label11 );
#endif

#ifdef __TEMPHUM6__
    TEMP = temphum6_readTemperature( _TEMPHUM6_TEMP_IN_CELSIUS );
    FloatToStr( TEMP, Temperature_str );
    decimal_trim( Temperature_str, 1 );
    DrawImage( &Image2 );
    Label7.Caption =  Temperature_str;
    DrawLabel( &Label7 );

    HUM = temphum6_readRelativeHuminidy( );
    FloatToStr( HUM, Humidity_str );
    decimal_trim( Humidity_str, 1 );
    DrawImage( &Image3 );
    Label8.Caption = Humidity_str;
    DrawLabel( &Label8 );
#endif

#ifdef __WATERDETECT__
    wdState = waterdetect_getStatus( );

    if ( wdState != wdStateOld )
    {
        if ( wdState > wdStateOld )
        {
           flag++;
        }
        if ( wdState < wdStateOld )
        {
           flag--;
        }
        wdStateOld = wdState;

        if ( flag <= 1 )
        {
            DrawImage( &Image9 );
        }
        else if ( flag == 2 )
        {
            DrawImage( &Image10 );
        }
        else if ( flag >= 3)
        {
           DrawImage( &Image11 );
        }
        DrawButton( &Button1 );
        DrawLabel( &Label14 );
    }
#endif

#ifdef __AIRQUALITY4__
    air_quality4_setBaseline( );

    air_quality4_getCO2AndTVOC( &data_buffer[ 0 ] );

    IntToStr(data_buffer[ 0 ], CO2_str);
    decimal_trim( CO2_str, 2 );
    DrawImage( &Image7 );
    Label12.Caption = CO2_str;
    DrawLabel( &Label12 );

    IntToStr(data_buffer[ 1 ], TVOC_str);
    decimal_trim( TVOC_str, 2 );
    DrawImage( &Image8 );
    Label13.Caption = TVOC_str;
    DrawLabel( &Label13 );
#endif

}

void InitTimer2( )
{
  RCC_APB1ENR.TIM2EN = 1;
  TIM2_CR1.CEN = 0;
  TIM2_PSC = 3999;
  TIM2_ARR = 62999;
  NVIC_IntEnable( IVT_INT_TIM2 );
  TIM2_DIER.UIE = 1;
  TIM2_CR1.CEN = 1;
  EnableInterrupts( );
}

void Timer2_interrupt( ) iv IVT_INT_TIM2
{
  TIM2_SR.UIF = 0;
  timer_flag = 1;
}

void main( )
{
  systemInit( );
  applicationInit( );

  Start_TP( );
  InitTimer2( );

  while ( 1 )
  {
    Check_TP( );

    if ( timer_flag == 1 )
    {
        applicationTask( );
        timer_flag = 0;
    }
    turn_fan_on_and_off( );
  }

}